%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%               INITIAL CONDITIONS FOR GENERATOR
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

load lfl.dat; %--Loading Load  Flow Data--
load sat.dat; %--Loading Saturation Data--

xd=gen(:,2)';
xdd=gen(:,3)';
xddd=gen(:,4)';
Td0d=gen(:,5)';
Td0dd=gen(:,6)';
xq=gen(:,7)';
xqd=gen(:,8)';
xqdd=gen(:,9)';
Tq0d=gen(:,10)';
Tq0dd=gen(:,11)';
H=gen(:,12)';
D=gen(:,13)';
xl=gen(:,14)';

model=2.2*(gt(xdd,xddd) & gt(xqd,xqdd))+2.1*((gt(xdd,xddd) & eq(xqd,xqdd)) & gt(xq,xqd));
model=model+ 2.0*(gt(xdd,xddd) & eq(xqdd,xq))+1.1*((eq(xdd,xddd) & ne(xdd,xq))& gt(xq,xqd));
model=model+1.0*((eq(xdd,xddd) & eq(xq,xqd)) & lt(Td0d,20))+0.0*((eq(xdd,xddd) & eq(xdd,xq))& gt(Td0d,100));

Vg0=lfl(gen(:,1),2)';
thetag0=pi/180*lfl(gen(:,1),3)';

Vg0bar=(Vg0.*cos(thetag0))+j*Vg0.*sin(thetag0);
Ig0bar=conj((lfl(gen(:,1),4)'+j*lfl(gen(:,1),5)')./Vg0bar);

%--------------Unsaturated Parameters Calculations-----------------

xadu=xd-xl;
xaqu=xq-xl;

ckl=lt(model,2.2)*1;
chl=le(model,1.1)*1;
cgl=(le(model,2.0) & ne(model,1.1))*1;

xfl=xadu.*(xdd-xl)./(xd-xdd);
xhl=10000*chl+bitcmp(chl,1).*(((xddd-xl).*xadu.*xfl)./((xadu.*xfl-(xfl+xadu).*(xddd-xl))+1e-8));
RF=ne(model,0.0).*((xadu+xfl)./(Td0d*wB));
RH=(xhl+(xadu.*xfl)./(xadu+xfl))./(Td0dd*wB);

xgl=10000*cgl+bitcmp(cgl,1).*(xaqu.*(xqd-xl)./((xq-xqd)+1e-8));
xkl=10000*ckl+bitcmp(ckl,1).*(((xqdd-xl).*xaqu.*xgl)./((xaqu.*xgl-(xgl+xaqu).*(xqdd-xl))+1e-8));
RG=(xaqu+xgl)./(Tq0d*wB);
RK=(xkl+(xaqu.*xgl)./(xaqu+xgl))./(Tq0dd*wB);

%---------End of Unsaturated Parameters Calculations----------


%-----------------  Saturation Calculations ------------------

Ksd=ones(1,ngen); 
Ksq=ones(1,ngen);

Ea=Vg0bar+j*xl.*Ig0bar;
siaT0=abs(Ea);

sidT1=sat(:,2)';
siaT1=sat(:,3)';
siaTu1=sat(:,4)';
siaT2=sat(:,5)';
siaTu2=sat(:,6)';

siId1=siaTu1-siaT1;
siId2=siaTu2-siaT2;
Bsd=(log(siId1)-log(siId2))./(siaT1-siaT2);
Asd=exp(log(siId2)-Bsd.*(siaT2-sidT1));

Asd_sat=zeros(1,nb);
Asd_sat(sat(:,1))=Asd;
Asd=Asd_sat(gen(:,1));

Bsd_sat=zeros(1,nb);
Bsd_sat(sat(:,1))=Bsd;
Bsd=Bsd_sat(gen(:,1));

siqT1=sat(:,7)';
siaT1=sat(:,8)';
siaTu1=sat(:,9)';
siaT2=sat(:,10)';
siaTu2=sat(:,11)';

siIq1=siaTu1-siaT1;
siIq2=siaTu2-siaT2;
Bsq=(log(siIq1)-log(siIq2))./(siaT1-siaT2);
Asq=exp(log(siIq2)-Bsq.*(siaT2-siqT1));

Asq_sat=zeros(1,nb);
Asq_sat(sat(:,1))=Asq;
Asq=Asq_sat(gen(:,1));

Bsq_sat=zeros(1,nb);
Bsq_sat(sat(:,1))=Bsq;
Bsq=Bsq_sat(gen(:,1));

sidT1_sat=zeros(1,nb);
sidT1_sat(sat(:,1))=sidT1;
sidT1=sidT1_sat(gen(:,1));

siqT1_sat=zeros(1,nb);
siqT1_sat(sat(:,1))=siqT1;
siqT1=siqT1_sat(gen(:,1));

%----------------Including Saturation Effect-----------------
f_sat=ones(1,nb);
f_nosat=[];      %Enter the Gen no. in [ ] for no saturation
f_sat(f_nosat)=0;    
f_sat=f_sat(gen(:,1));
%------------------------------------------------------------

%----------------Including Saliency Effect-------------------
f_sil=ones(1,nb);
f_qed=[];      %Enter the Gen no. in [ ] for no saliency
f_sil(f_qed)=0;
f_sil=f_sil(gen(:,1));
%------------------------------------------------------------

Asd=Asd.*f_sat.*ne(model,0.0);
Asq=Asq.*f_sat.*ne(model,0.0);
Asq=Asq.*f_sil+Asd.*bitcmp(f_sil,1);
Bsq=Bsq.*f_sil+Bsd.*bitcmp(f_sil,1);
siqT1=siqT1.*f_sil+sidT1.*bitcmp(f_sil,1);

Asd1=Asd.*gt(siaT0,sidT1);
Asq1=Asq.*gt(siaT0,siqT1);

siId=Asd1.*exp(Bsd.*(siaT0-sidT1));
siIq=Asq1.*exp(Bsq.*(siaT0-siqT1));

Ksd=siaT0./(siaT0+siId);
Ksq=siaT0./(siaT0+siIq);

xad=Ksd.*xadu;
xaq=Ksq.*xaqu;

xd=xad+xl;
xq=xaq+xl;

xaddd=1./((1./xad)+(1./xfl)+(1./xhl));
xaqdd=1./((1./xaq)+(1./xgl)+(1./xkl));

xddd=xaddd+xl;
xqdd=xaqdd+xl;
%-------------End of Saturation Calculations----------------

Eqbar=(Vg0bar+j*xq.*Ig0bar);
Eq=abs(Eqbar);
delta0=angle(Eqbar);

VQg0=real(Vg0bar);
VDg0=imag(Vg0bar);

iDg0=imag(Ig0bar);
iQg0=real(Ig0bar);

Ig0bardq=Ig0bar.*(cos(delta0)-j*sin(delta0));
iq0=real(Ig0bardq);
id0=imag(Ig0bardq);

Vg0bardq=Vg0bar.*(cos(delta0)-j*sin(delta0));
vqg0=real(Vg0bardq);
vdg0=imag(Vg0bardq);

%-----------------d and q axes States--------------------

sid0=vqg0;
siq0=-vdg0;

ifd0=(sid0-xd.*id0)./xad;
Efd0=xadu.*ifd0;
IFD0=xadu.*ifd0;

siad0=sid0-xl.*id0;
siaq0=siq0-xl.*iq0;

sif0=siad0 + (xfl./xadu).*Efd0;
sih0=siad0;

sig0=siaq0;
sik0=siaq0;

%--------------------------------------------------------

%----------------------Torque----------------------------
Eqdd0=xaddd.*((sif0./xfl) + (sih0./xhl));
Eddd0=-xaqdd.*((sig0./xgl) + (sik0./xkl));
Tm0=Eqdd0.*iq0+Eddd0.*id0+id0.*iq0.*(xaddd-xaqdd);
%--------------------------------------------------------

%-------------------Dummy coil---------------------------
Edummydd0=-(xqdd-xddd).*iq0;
Tdummy=0.01;
%--------------------------------------------------------

[mm order_gen]=sort([setxor(1:nb,gen(:,1));gen(:,1)]);

%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%Exciter Initial Condition calculations
EFD01=[zeros(1, nb-ngen) Efd0];
EFD0=EFD01(order_gen);
IFD01=[zeros(1, nb-ngen) IFD0];
IFD0_a=IFD01(order_gen);
Vg0bar1=[zeros(1, nb-ngen) Vg0bar];
Vg0bar_a=Vg0bar1(order_gen);
Ig0bar1=[zeros(1, nb-ngen) Ig0bar];
Ig0bar_a=Ig0bar1(order_gen);
%----------------------Static type (modified)exciters------------------
static_exciter
%----------------------Static -ST1A type exciters------------------
ST1A_exciter
%-------------------Compound source rectifier exciter: ST2A TYPE----------------
ST2A_exciter
%----------------------Alternator supplied Controlled Rectifier: AC4A type-
AC4A_exciter
%------------------------AC1A Exciters-------------------
AC1A_exciter
%------------------------DC1A (Type-1) Exciters-------------------
DC1A_exciter

%-----------Main Selector for excitation systems.
%AVR = 1's (DISABLED); =0's (ENABLED)
%AVR=ones(1,nb);
%AVR=zeros(1,nb);
AVR =[0 0 0 1];

SL_static(1:nb)=1; SL_ST1A(1:nb)=1; SL_ST2A(1:nb)=1; SL_AC1A(1:nb)=1; 
SL_AC4A(1:nb)=1; SL_DC1A(1:nb)=1;

%-----Indicate the generator number on which a specfic type of 
%------- exciter is to be enabled, otherwise null
ng_static=[1];
ng_ST1A=[];
ng_ST2A=[];
ng_AC1A=[];
ng_AC4A=[];
ng_DC1A=[];

SL_static(ng_static)=0;
SL_ST1A(ng_ST1A)=0;
SL_ST2A(ng_ST2A)=0;
SL_AC1A(ng_AC1A)=0;
SL_AC4A(ng_AC4A)=0;
SL_DC1A(ng_DC1A)=0;

%-------------------Load Modelling------------------
% Load varables
VL0=lfl(ld(:,1),2);
theL0=lfl(ld(:,1),3)*pi/180;
PL0=ld(:,2);
QL0=ld(:,3);
% set ZIP model coefficients in file *.m
load_zip_model

%------Turbine and Speed Governers- Initial condition calculations.
H1=[zeros(1, nb-ngen) H];
H_a=H1(order_gen);

Tm01=[zeros(1, nb-ngen) Tm0];
TmA0=Tm01(order_gen);

%speed Governer system with Hydro Turbine.
hydro_turbine

%Speed governer with nonreheat type turbine
NonReheat_turbine

%Speed governer with reheat type turbine

Reheat_turbine

% ---------------------Main Selector for Turbine models---------------
%TURB = 1's (DISABLED); =0's (ENABLED)
TURB=ones(1,nb);
%TURB=zeros(1,nb);

SL_HYDRO(1:nb)=1; SL_NRST(1:nb)=1; SL_RHST(1:nb)=1;

%-----Indicate the generator number on which a specfic type of 
%------- speed-governor-turbine is to be enabled, otherwise null
ng_hydro=[];
ng_nrt=[];
ng_rht=[];

SL_HYDRO(ng_hydro)=0;
SL_NRST(ng_nrt)=0;
SL_RHST(ng_rht)=0;

%---------------PSS models----------------------
%------Slip signal based PSS----
pss_slip_signal

%----Power signal based PSS----
pss_power_signal

%----Frequency signal based PSS----
pss_freq_signal

%----Delta Power-Omega signal based PSS----
pss_delPw_signal

%---Main Selector for PSS----
%PSS = 1's (DISABLED); =0's (ENABLED)
%PSS=ones(1,nb);
PSS=zeros(1,nb);

%----Selectors for individual type of PSS----
SL_slip_pss(1:nb)=1; SL_power_pss(1:nb)=1; SL_freq_pss(1:nb)=1;
SL_delPw_pss(1:nb)=1;

%-----Indicate the generator number on which a specfic type of 
%------- PSS is to be enabled, otherwise nullmatrix---

ng_slip_pss=[1];
ng_power_pss=[];
ng_freq_pss=[];
ng_delPw_pss=[];

SL_slip_pss(ng_slip_pss)=0;
SL_power_pss(ng_power_pss)=0;
SL_freq_pss(ng_freq_pss)=0;
SL_delPw_pss(ng_delPw_pss)=0;

%-------------------For checking models at the time of builbing blocks
busang=lfl(:,3)*pi/180;
Vpre=lfl(:,2).*(cos(busang)+j*sin(busang));
%-----------------------------------------------------
